@@ -54,7 +54,7 @@ from sklearn.model_selection import StratifiedKFold
 from sklearn.metrics import mean_absolute_error
 from sklearn.metrics import accuracy_score
 from sklearn.metrics import log_loss
-from sklearn.externals import six
+from sklearn.metrics import mean_squared_error
 
 # -----------------------------------------------------------------------------
 # -----------------------------------------------------------------------------
@@ -851,6 +851,44 @@ class StackingTransformer(BaseEstimator, TransformerMixin):
     # -------------------------------------------------------------------------
     # -------------------------------------------------------------------------
 
+    def _random_choice(self, n, size, bound=2**30):
+        """
+        Memory efficient substitute for np.random.choice without replacement
+
+        Parameters:
+        ===========
+        n : int
+            Upper value for range to chose from: [0, n).
+            This parameter is bounded (see bound).
+        size: int
+            Number of values to chose
+        bound : int
+            Upper random int for backward compatibility
+            with some older numpy versions
+
+        Returns:
+        ========
+        ids : 1d numpy array of shape (size, ) dtype=np.int32
+        """
+        try:
+            if n < size:
+                raise ValueError('Drawing without replacement: '
+                                 '``n`` cannot be less than ``size``')
+
+            ids = []
+            while len(ids) < size:
+                rnd = np.random.randint(min(bound, n))
+                if rnd not in ids:
+                    ids.append(rnd)
+            return np.array(ids, dtype=np.int32)
+
+        except Exception:
+            raise ValueError('Internal error. '
+                             'Please save traceback and inform developers.')
+
+    # -------------------------------------------------------------------------
+    # -------------------------------------------------------------------------
+
     def _get_footprint(self, X, n_items=1000):
         """Selects ``n_items`` random elements from 2d numpy array or
         sparse matrix (or all elements if their number is less or equal
@@ -861,7 +899,11 @@ class StackingTransformer(BaseEstimator, TransformerMixin):
             r, c = X.shape
             n = r * c
             # np.random.seed(0) # for development
-            ids = np.random.choice(n, min(n_items, n), replace=False)
+
+            # OOM with large arrays (see #29)
+            # ids = np.random.choice(n, min(n_items, n), replace=False)
+
+            ids = self._random_choice(n, min(n_items, n))
 
             for i in ids:
                 row = i // c
@@ -913,7 +955,7 @@ class StackingTransformer(BaseEstimator, TransformerMixin):
             return out
         out.update(estimators)
         for name, estimator in estimators:
-            for key, value in six.iteritems(estimator.get_params(deep=True)):
+            for key, value in iter(estimator.get_params(deep=True).items()):
                 out['%s__%s' % (name, key)] = value
         return out
 
